require_relative '../lib/shoestrap/application_generator'

def generate_test_app_with_shoestrap(options=nil)
  ENV['SHOESTRAP_PATH'] = File.join(File.dirname(__FILE__), '..')

  if options.is_a?(Hash)
    aggregated_options = []
    options.each_pair {|k,v| aggregated_options << "--#{k}=#{v}" }
    options = aggregated_options.join(' ')
    aggregated_options = nil
  end

  app_identifier = test_app_key(options)
  puts if debug_shoestrap?
  dbg_comment "[ app with options #{options.inspect} will be known as #{app_identifier.inspect} ]"
  dbg_comment "[ app without any options would be known as #{test_app_key(nil).inspect} ]" unless options.blank?
  if Postgres.create_user app_identifier
    dbg_comment "successfully created postgres user #{app_identifier.inspect}"
  else
    dbg_comment "Won't create user #{app_identifier.inspect} while he already exists."
  end

  app_path = test_app_path(options)
  result = nil

  if target_app_path || File.exists?(app_path)
    if File.exists?(app_path)
      dbg_comment "Application path exists. I will just create the databases now. Pay attention if you have a task that makes something database related."
      Dir.chdir(app_path) do
        reload_application_gemfile options
        Postgres.create_user app_identifier

        # destroy all scaffolds
        Dir['app/models/*.rb','app/models/cms/*.rb'].each do |path|
          model_name = File.basename(path).gsub(File.extname(path),'')
          drop_shoestrap_scaffold_for(model_name, options) unless model_name == 'admin'
        end

        result = bundle_exec 'rake db:drop db:create db:migrate db:seed'
      end
    end
  else
    dbg_comment "generating test app in #{app_path}"
    result = bundle_exec shoestrap_new(options)
  end
  puts if debug_shoestrap?
  result
end

def shoestrap_new(options = nil)
  "#{shoestrap_bin_path} new #{test_app_path(options)} #{options}"
end

[:rails, :rake].each do |command|
  define_method command do |task, options = nil|
    app_path = test_app_path(options)
    Dir.chdir(app_path) do
      dbg_comment "cd #{app_path}"
      reload_application_gemfile options
      bundle_exec "#{command} #{task}"
    end
  end
end

def bundle_exec(command)
  if debug_shoestrap?
    puts "#{ts} $ bundle exec #{command}"
    outstr, errstr, status = Open3.capture3('bundle', 'exec', command)

    # a²: why are info-messages in error out?
    errstr = errstr.split("\n").select{|l| !l.blank? && !(l =~ /^\s*\d\d?:\d\d?:\d\d? - INFO -/) }.join("\n")

    if errstr.present?
      puts "there where errors while executing the command"
      puts "\e[0;31m"+errstr.strip+"\e[0m"
      puts
      caller.select{|cl| !cl =~ /\/ruby\/gems\// }.each{|called_line| puts called_line }
      puts
    end
    outstr
    # dbg_comment "command #{command.inspect} finished"
  else
    `bundle exec #{command}`
  end
end

def dbg_comment( msg )
  return unless debug_shoestrap?
  puts " # #{ts} - #{msg}"
end

def ts
  Time.now.strftime('%H:%M:%S')
end
