[
  'rspec', 'fileutils', 'digest', 'pry', 'open3'
].each {|lib| require lib }

[
  './postgres_helper', './custom_matcher', './command_wrapper_helper'
].each {|lib| require_relative lib }

# TODO: spec these helpers!

JUST_GENERATE_NEW_APP_IF_THESE_ARE_CHANGED = [
  'lib',
  'templates'
]

def filetree_fingerprint(dirlist = JUST_GENERATE_NEW_APP_IF_THESE_ARE_CHANGED)
  aggregated_hashes = ''
  [dirlist].flatten.sort.each do |dir|
    Dir[File.join(shoestrap_path, dir, '**/**')].sort.each do |f|
      aggregated_hashes << (File.directory?(f) ? filetree_fingerprint(f) : Digest::MD5.hexdigest(File.read f).to_s)
    end
  end
  Digest::MD5.hexdigest(aggregated_hashes).to_s
end

def reload_application_gemfile(options=nil)
  gemfile_path = File.join(application_path(options), 'Gemfile')
  # dbg_comment "loading Gemfile #{gemfile_path.inspect}; given options #{options.inspect}"
  ENV['BUNDLE_GEMFILE'] = gemfile_path
end

def clean_up
  puts
  puts 'cleaning up...'
  remove_test_dbs
  drop_database_from_dummy_apps
  remove_test_apps if on_ci_system?
  ENV.delete 'SHOESTRAP_PATH'
  puts
  puts
end

def drop_database_from_dummy_apps
  Postgres.drop_users_like 'test_app%'
end

def test_apps
  Dir.entries(File.join(File.dirname(__FILE__))).select do |entry|
    entry.include? 'test_app'
  end.map { |e| "features/#{e}" }
end

def remove_test_apps
  test_apps.each do |app|
    puts "removing test app #{File.expand_path(app)}"
    FileUtils.rm_rf(app)
  end
end

def remove_test_dbs
  test_apps.each do |app|
    Dir.chdir app do
      next unless Postgres.database_exists?(app.split('/').last + '_test')
      puts "dropping test db for #{app}" if debug_shoestrap?
      ENV['BUNDLE_GEMFILE'] = File.join(Dir.pwd, 'Gemfile')
      require 'bundler'
      Bundler.require(:default)
      bundle_exec 'rake db:drop'
    end
  end
  Postgres.drop_databases_like('test_app_%')
end

def drop_shoestrap_scaffold_for(name, options = nil)
  %w(production development test).each do |db_env|
    app_path = test_app_path(options)
    unless File.exists? app_path
      dbg_comment "won't destroy scaffold for #{name.inspect} while app #{app_path} doesn't exist."
      return
    end

    Dir.chdir(app_path) do
      database_name = app_path.split('/').last
      rails "destroy scaffold #{name}", options
      rails "destroy scaffold cms/#{name}", options
      database = "#{database_name}_#{db_env}"
      next unless Postgres.database_exists? database
      # Postgres.select_database database
      Postgres.drop_table database_name, database, name
    end
  end
end

def application_path(options = nil)
  target_app_path || test_app_path(options)
end

def shoestrap_path
  File.expand_path(File.join(File.dirname(__FILE__),'../'))
end

def shoestrap_bin_path
  File.join(shoestrap_path, 'bin/shoestrap')
end

def target_app_path
  ENV['TARGET_APP']
end

def test_app_path(options = nil)
  File.join(File.dirname(__FILE__), test_app_key(options))
end

def application_file(file, options = nil)
  File.read(File.join(application_path(options), file))
end

def test_app_key(options)
  app_hash = options.to_s.strip.gsub(/\W+/,'') + filetree_fingerprint
  # a²: we need to limit the size due to maximum database name sizes in postgres
  "test_app_#{(app_hash.size > 42) ? Digest::MD5.hexdigest(app_hash) : app_hash}"[0..42]
end

def debug_shoestrap?; !!ENV['DEBUG'] end

def on_ci_system?
  ENV['TRAVIS'] || ENV['CI'] || ENV['CIRCLECI']
end

RSpec.configure do |config|
  config.include(FeatureMatcher)
  config.after(:suite) do
    clean_up
  end
  config.before(:suite) do
    remove_test_apps if ENV['REMOVE_CACHE_AT_FIRST']
  end
end
