require_relative '../../features_helper'

describe 'rails generate shoestrap:admin' do

  before :all do
    generate_test_app_with_shoestrap
    rails 'generate shoestrap:admin'
  end

  let(:app_path){ application_path }

  it 'generates an admin cms model' do
    expect(application_file('app/models/admin.rb')).not_to be_nil
    expect(Dir.glob("#{app_path}/db/migrate/*create*admins*").size).to be 1
    expect(application_file('config/routes.rb')).to match(/resources :admins/)
  end

  it 'installs devise' do
    expect(application_file('Gemfile')).to match('devise')
    expect(application_file('config/initializers/devise.rb')).not_to be_nil
  end

  it 'disables the devise registerable module' do
    expect(application_file('app/models/admin.rb')).not_to match(':registerable')
  end

  it 'configures devise to use the devise layout' do
    expect(application_file('config/initializers/devise.rb')).to match("devise_layout = 'devise'")
  end

  it 'adds a before filter to kuhsaft controller' do
    expect(application_file('config/initializers/kuhsaft.rb')).to match('before_filter :authenticate_admin!')
  end

  it 'adds a before filter to shoestrap base controller' do
    expect(application_file('app/controllers/shoestrap/base_controller.rb')).to match('before_filter :authenticate_admin!')
  end

  it 'installs the devise layout' do
    expect(application_file('app/views/layouts/devise.html.haml')).not_to be_nil
  end

  it 'installs the devise views' do
    expect(application_file('app/views/admins/sessions/new.html.haml')).not_to be_nil
  end

  it 'installs the devise translations' do
    expect(application_file('config/locales/de/admin.yml')).not_to be_nil
  end
end
