require_relative '../../features_helper'

describe 'application generated with shoestrap new <name> --kuhsaft true' do

  options = {kuhsaft: true}

  let(:kuhsaft_migrations_count) { 14 }
  let(:app_path){application_path(options)}
  let(:migration_path) { File.join(app_path,'db/migrate/') }
  let(:migrations_count) { Dir.glob(File.join(migration_path, '*kuhsaft*.rb')).size }

  before :all do
    generate_test_app_with_shoestrap(options)
  end

  it 'has the kuhsaft migrations installed' do
    expect(migrations_count).to be kuhsaft_migrations_count
  end

  it 'has db migrated with kuhsaft migrations' do
    reload_application_gemfile(options)
    migration_status = Dir.chdir(app_path) { rake 'db:migrate:status', options }
    expect(migration_status.split("\n").select { |x| x.include? 'kuhsaft' }.size).to be kuhsaft_migrations_count
  end

  it 'has the db with kuhsaft seeds' do
    expect(Dir.chdir(app_path) { rails 'runner "puts Kuhsaft::BrickType.count"', options }.to_i).to be 11
  end

  it 'has the kuhsaft assets installed' do
    expect(application_file('app/assets/stylesheets/kuhsaft/cms/customizations.css.sass', options)).not_to be_nil
    expect(application_file('app/assets/javascripts/kuhsaft/cms/customizations.js.coffee', options)).not_to be_nil
    expect(application_file('app/assets/javascripts/kuhsaft/cms/ck-config.js.coffee', options)).not_to be_nil
  end

  it 'has kuhsaft mounted as an engine on /' do
    expect(application_file('config/routes.rb', options)).to match("mount Kuhsaft::Engine => '/'")
  end

  it 'has the default image size initializer' do
    expect(application_file('config/initializers/kuhsaft.rb', options)).not_to be_nil
  end

  it 'generates a translation file for the available locales' do
    translation_migrations = Dir[migration_path+'*add_*_translation.rb']

    (I18n.available_locales - [:de, :en]).each do |locale|
      matched_migration = translation_migrations.detect{|m| m.include?("add_#{locale}_translation.rb") }
      expect(matched_migration).not_to be_nil
    end
  end

  it 'makes the kuhsaft helpers available in the host app' do
    expect(application_file('app/controllers/application_controller.rb')).to match("helper Kuhsaft::Engine.helpers")
  end
end
